-- Custom Behavior Helpers --

-- Copied from the Star Road Port (thx to who made the function)
--- Replacement for DROP_TO_FLOOR()
function object_drop_to_floor(obj)
    local x = obj.oPosX
    local y = obj.oPosY
    local z = obj.oPosZ

    local floorHeight = find_floor_height(x, y + 200, z)
    obj.oPosY = floorHeight
    obj.oMoveFlags = (obj.oMoveFlags | OBJ_MOVE_ON_GROUND)
end

-- Custom Behaviors --

local function anim_replace(o)
    if obj_has_behavior_id(o, id_bhvWigglerHead) then
        o.oAnimations = gObjectAnimations.piranha_plant_seg6_anims_0601C31C
    end
end

hook_behavior(id_bhvWigglerHead, OBJ_LIST_GENACTOR, false, anim_replace, nil)
hook_behavior(id_bhvWigglerBody, OBJ_LIST_UNIMPORTANT, false, obj_mark_for_deletion, nil)

local function goomba_npc_init(o)
    o.oFlags = OBJ_FLAG_COMPUTE_ANGLE_TO_MARIO | OBJ_FLAG_HOLDABLE | OBJ_FLAG_COMPUTE_DIST_TO_MARIO | OBJ_FLAG_SET_FACE_YAW_TO_MOVE_YAW | OBJ_FLAG_UPDATE_GFX_POS_AND_ANGLE
    o.oAnimations = gObjectAnimations.goomba_seg8_anims_0801DA4C
    o.oInteractType = INTERACT_TEXT
    object_drop_to_floor(o)
    obj_set_hitbox_radius_and_height(o, 100, 60)
    obj_init_animation(o, 0)
    o.oBobombBuddyRole = 0
    cur_obj_set_home_once()
    bhv_bobomb_buddy_init()
    o.oIntangibleTimer = 0
end

local function goomba_npc_loop(o)
    bhv_bobomb_buddy_loop() -- Generates script errors if used as the loop function for some reason
end

hook_behavior(id_bhvJrbSlidingBox, OBJ_LIST_GENACTOR, true, goomba_npc_init, goomba_npc_loop)

local function small_thwomp_init(o)
    o.oFlags = OBJ_FLAG_HOLDABLE | OBJ_FLAG_COMPUTE_DIST_TO_MARIO | OBJ_FLAG_SET_FACE_YAW_TO_MOVE_YAW | OBJ_FLAG_UPDATE_GFX_POS_AND_ANGLE
    o.collisionData = gGlobalObjectCollisionData.thwomp_seg5_collision_0500B7D0
    object_drop_to_floor(o)
    o.oPosY = o.oPosY + 1
    cur_obj_scale(0.5)
    cur_obj_set_home_once()
    o.oDrawingDistance = 4000
    bhv_breakable_box_small_init()
    o.oIntangibleTimer = 0
end

local function small_thwomp_loop(o)
    load_object_collision_model()
    bhv_breakable_box_small_loop()
    bhv_grindel_thwomp_loop()
end

hook_behavior(id_bhvWfBreakableWallRight, OBJ_LIST_DESTRUCTIVE, true, small_thwomp_init, small_thwomp_loop)

jpChainChomp = audio_sample_load("jp_chain_chomp.ogg")

local function jp_chain_chomp_sfx_mute(sfx)
    if sfx == SOUND_GENERAL_CHAIN_CHOMP2 then
        audio_sample_play(jpChainChomp, gMarioStates[0].pos, 3)
        return 0
    end
end

hook_event(HOOK_ON_PLAY_SOUND, jp_chain_chomp_sfx_mute)

--[[
local function jp_chain_chomp_sfx_replace(o)
    if o.oSubAction == CHAIN_CHOMP_SUB_ACT_TURN and o.oTimer >= 40 then
        audio_sample_play(jpChainChomp, o.header.gfx.pos, 5)
    end
end

hook_behavior(id_bhvChainChomp, OBJ_LIST_GENACTOR, false, nil, jp_chain_chomp_sfx_replace)]]

local function moving_flame_init(o)
    o.oFlags = OBJ_FLAG_UPDATE_GFX_POS_AND_ANGLE
    obj_set_billboard(o)
    cur_obj_set_home_once()
    cur_obj_scale(5.0)
    local hitbox = get_temp_object_hitbox()
    hitbox.interactType = INTERACT_FLAME
    hitbox.radius = 50/5
    hitbox.height = 25/5
    hitbox.downOffset = 25/5
    obj_set_hitbox(o, hitbox)
    --cur_obj_set_hitbox_radius_and_height(50, 25)
    o.oIntangibleTimer = 0
    bhv_init_room()
end

local function moving_flame_loop(o)
    o.oInteractStatus = 0
    o.oAnimState = o.oAnimState + 1
    o.oForwardVel = math.sin(o.oTimer/5) * 75
    cur_obj_move_using_fvel_and_gravity()
    --djui_chat_message_create(tostring(o.oForwardVel))
end

hook_behavior(id_bhvBowserSubDoor, OBJ_LIST_LEVEL, true, moving_flame_init, moving_flame_loop)

--[[
function test()
    local m = gMarioStates[0]
    if m.controller.buttonPressed & D_JPAD ~= 0 then
        m.pos.x = -1655
        m.pos.y = 1774
        m.pos.z = -4526
        m.faceAngle.y = 16384
    end
end

hook_event(HOOK_UPDATE, test)]]